@@ -19,32 +19,17 @@
  * Boston, MA 02110-1301, USA.
  */
 
-#ifndef __GUPNP_ACL_H__
-#define __GUPNP_ACL_H__
+#ifndef GUPNP_ACL_H
+#define GUPNP_ACL_H
 
 #include <glib.h>
 #include <glib-object.h>
-#include <gio/gio.h>
+#include <libsoup/soup-session.h>
 
 G_BEGIN_DECLS
 
-GType
-gupnp_acl_get_type (void) G_GNUC_CONST;
-
 #define GUPNP_TYPE_ACL (gupnp_acl_get_type())
-
-#define GUPNP_ACL(obj)                  \
-    (G_TYPE_CHECK_INSTANCE_CAST ((obj), \
-     GUPNP_TYPE_ACL,                    \
-     GUPnPAcl))
-
-#define GUPNP_IS_ACL(obj)               \
-    (G_TYPE_CHECK_INSTANCE_TYPE ((obj),  \
-     GUPNP_TYPE_ACL))
-
-#define GUPNP_ACL_GET_INTERFACE(obj)      \
-    (G_TYPE_INSTANCE_GET_INTERFACE ((obj), \
-     GUPNP_TYPE_ACL, GUPnPAclInterface))
+G_DECLARE_INTERFACE (GUPnPAcl, gupnp_acl, GUPNP, ACL, GObject)
 
 /**
  * GUPnPAcl:
@@ -135,4 +120,4 @@ gupnp_acl_can_sync (GUPnPAcl *self);
 
 G_END_DECLS
 
-#endif
+#endif /* GUPNP_ACL_H */
