@@ -34,6 +34,8 @@
  * Since: 0.13.0
  */
 
+#define G_LOG_DOMAIN "gupnp-context-manager"
+
 #include <config.h>
 #include <errno.h>
 #include <string.h>
@@ -44,7 +46,6 @@
 #include <glib/gstdio.h>
 
 #include "gupnp.h"
-#include "gupnp-marshal.h"
 
 #ifdef HAVE_IFADDRS_H
 #include "gupnp-unix-context-manager.h"
@@ -56,14 +57,13 @@
 #include "gupnp-network-manager.h"
 #elif defined(USE_CONNMAN)
 #include "gupnp-connman-manager.h"
+#elif defined(USE_NETLINK)
+#include "gupnp-linux-context-manager.h"
 #endif
 
-G_DEFINE_ABSTRACT_TYPE (GUPnPContextManager,
-                        gupnp_context_manager,
-                        G_TYPE_OBJECT);
-
 struct _GUPnPContextManagerPrivate {
         guint              port;
+        GSocketFamily      family;
 
         GUPnPContextManager *impl;
 
@@ -72,11 +72,16 @@ struct _GUPnPContextManagerPrivate {
 
         GUPnPWhiteList *white_list;
 };
+typedef struct _GUPnPContextManagerPrivate GUPnPContextManagerPrivate;
+
+G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (GUPnPContextManager,
+                                     gupnp_context_manager,
+                                     G_TYPE_OBJECT);
 
 enum {
         PROP_0,
-        PROP_MAIN_CONTEXT,
         PROP_PORT,
+        PROP_SOCKET_FAMILY,
         PROP_WHITE_LIST
 };
 
@@ -94,8 +99,11 @@ on_context_available (GUPnPContextManager    *manager,
                       G_GNUC_UNUSED gpointer *user_data)
 {
         GUPnPWhiteList *white_list;
+        GUPnPContextManagerPrivate *priv;
 
-        white_list = manager->priv->white_list;
+        priv = gupnp_context_manager_get_instance_private (manager);
+
+        white_list = priv->white_list;
 
         /* Try to catch the notification, only if the white list
          * is enabled, not empty and the context doesn't match */
@@ -110,9 +118,8 @@ on_context_available (GUPnPContextManager    *manager,
                 g_object_set (context, "active", FALSE, NULL);
 
                 /* Save it in case we need to re-enable it */
-                manager->priv->blacklisted = g_list_prepend (
-                                                manager->priv->blacklisted,
-                                                g_object_ref (context));
+                priv->blacklisted = g_list_prepend (priv->blacklisted,
+                                                    g_object_ref (context));
         }
 }
 
@@ -123,12 +130,15 @@ on_context_unavailable (GUPnPContextManager    *manager,
 {
         GList *l;
         GList *black;
+        GUPnPContextManagerPrivate *priv;
+
+        priv = gupnp_context_manager_get_instance_private (manager);
 
         /* Make sure we don't send anything on now unavailable network */
         g_object_set (context, "active", FALSE, NULL);
 
         /* Unref all associated objects */
-        l = manager->priv->objects;
+        l = priv->objects;
 
         while (l) {
                 GUPnPContext *obj_context = NULL;
@@ -152,22 +162,22 @@ on_context_unavailable (GUPnPContextManager    *manager,
 
                         g_object_unref (l->data);
 
-                        manager->priv->objects =
-                                g_list_delete_link (manager->priv->objects, l);
+                        priv->objects = g_list_delete_link (priv->objects,
+                                                            l);
                         l = next;
                 } else {
                         l = l->next;
                 }
         }
 
-        black = g_list_find (manager->priv->blacklisted, context);
+        black = g_list_find (priv->blacklisted, context);
 
         if (black != NULL) {
                 g_signal_stop_emission_by_name (manager, "context-unavailable");
 
                 g_object_unref (black->data);
-                manager->priv->blacklisted =
-                        g_list_delete_link (manager->priv->blacklisted, black);
+                priv->blacklisted = g_list_delete_link (priv->blacklisted,
+                                                        black);
         }
 }
 
@@ -180,22 +190,20 @@ gupnp_context_manager_filter_context (GUPnPWhiteList *white_list,
         GList *obj;
         GList *blk;
         gboolean match;
+        GUPnPContextManagerPrivate *priv;
 
-        obj = manager->priv->objects;
-        blk = manager->priv->blacklisted;
+        priv = gupnp_context_manager_get_instance_private (manager);
+
+        obj = priv->objects;
+        blk = priv->blacklisted;
 
         while (obj != NULL) {
                 /* If the white list is empty, treat it as disabled */
                 if (check) {
                         GUPnPContext *context;
-                        const char *property = "context";
-
-                        if (GUPNP_IS_CONTROL_POINT (obj->data)) {
-                                property = "client";
-                        }
 
                         g_object_get (G_OBJECT (obj->data),
-                                      property, &context,
+                                      "context", &context,
                                       NULL);
 
                         match = gupnp_white_list_check_context (white_list,
@@ -242,9 +250,8 @@ gupnp_context_manager_filter_context (GUPnPWhiteList *white_list,
                 g_signal_emit_by_name (manager, "context-available", blk->data);
 
                 g_object_unref (blk->data);
-                manager->priv->blacklisted = g_list_delete_link (
-                                                manager->priv->blacklisted,
-                                                blk);
+                priv->blacklisted = g_list_delete_link (priv->blacklisted,
+                                                        blk);
                 blk = next;
         }
 }
@@ -288,17 +295,16 @@ on_white_list_enabled_cb (GUPnPWhiteList *white_list,
 static void
 gupnp_context_manager_init (GUPnPContextManager *manager)
 {
-        manager->priv =
-                G_TYPE_INSTANCE_GET_PRIVATE (manager,
-                                             GUPNP_TYPE_CONTEXT_MANAGER,
-                                             GUPnPContextManagerPrivate);
+        GUPnPContextManagerPrivate *priv;
 
-        manager->priv->white_list = gupnp_white_list_new ();
+        priv = gupnp_context_manager_get_instance_private (manager);
 
-        g_signal_connect_after (manager->priv->white_list, "notify::entries",
+        priv->white_list = gupnp_white_list_new ();
+
+        g_signal_connect_after (priv->white_list, "notify::entries",
                                 G_CALLBACK (on_white_list_change_cb), manager);
 
-        g_signal_connect_after (manager->priv->white_list, "notify::enabled",
+        g_signal_connect_after (priv->white_list, "notify::enabled",
                                 G_CALLBACK (on_white_list_enabled_cb), manager);
 }
 
@@ -312,18 +318,14 @@ gupnp_context_manager_set_property (GObject      *object,
         GUPnPContextManagerPrivate *priv;
 
         manager = GUPNP_CONTEXT_MANAGER (object);
-        priv = manager->priv;
+        priv = gupnp_context_manager_get_instance_private (manager);
 
         switch (property_id) {
         case PROP_PORT:
                 priv->port = g_value_get_uint (value);
                 break;
-        case PROP_MAIN_CONTEXT:
-                if (g_value_get_pointer (value) != NULL)
-                        g_warning ("GUPnPContextManager:main-context is "
-                                   "deprecated. Use "
-                                   "g_main_context_push_thread_default()"
-                                   "instead.");
+        case PROP_SOCKET_FAMILY:
+                priv->family = g_value_get_enum (value);
                 break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -338,22 +340,20 @@ gupnp_context_manager_get_property (GObject    *object,
                                     GParamSpec *pspec)
 {
         GUPnPContextManager *manager;
+        GUPnPContextManagerPrivate *priv;
 
         manager = GUPNP_CONTEXT_MANAGER (object);
+        priv = gupnp_context_manager_get_instance_private (manager);
 
         switch (property_id) {
         case PROP_PORT:
-                g_value_set_uint (value, manager->priv->port);
+                g_value_set_uint (value, priv->port);
                 break;
-        case PROP_MAIN_CONTEXT:
-                g_warning ("GUPnPContextManager:main-context is deprecated. "
-                           "Use g_main_context_push_thread_default()"
-                           "instead.");
-                g_value_set_pointer (value,
-                                     g_main_context_get_thread_default ());
+        case PROP_SOCKET_FAMILY:
+                g_value_set_enum (value, priv->family);
                 break;
         case PROP_WHITE_LIST:
-                g_value_set_object (value, manager->priv->white_list);
+                g_value_set_object (value, priv->white_list);
                 break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -367,9 +367,12 @@ gupnp_context_manager_dispose (GObject *object)
         GUPnPContextManager *manager;
         GUPnPWhiteList *wl;
         GObjectClass *object_class;
+        GUPnPContextManagerPrivate *priv;
 
         manager = GUPNP_CONTEXT_MANAGER (object);
-        wl = manager->priv->white_list;
+        priv = gupnp_context_manager_get_instance_private (manager);
+
+        wl = priv->white_list;
 
         g_signal_handlers_disconnect_by_func (wl,
                                               on_white_list_enabled_cb,
@@ -379,14 +382,14 @@ gupnp_context_manager_dispose (GObject *object)
                                               on_white_list_change_cb,
                                               NULL);
 
-        g_list_free_full (manager->priv->objects, g_object_unref);
-        manager->priv->objects = NULL;
-        g_list_free_full (manager->priv->blacklisted, g_object_unref);
-        manager->priv->blacklisted = NULL;
+        g_list_free_full (priv->objects, g_object_unref);
+        priv->objects = NULL;
+        g_list_free_full (priv->blacklisted, g_object_unref);
+        priv->blacklisted = NULL;
 
         if (wl) {
                 g_object_unref (wl);
-                manager->priv->white_list = NULL;
+                priv->white_list = NULL;
         }
 
         /* Call super */
@@ -405,29 +408,6 @@ gupnp_context_manager_class_init (GUPnPContextManagerClass *klass)
         object_class->get_property = gupnp_context_manager_get_property;
         object_class->dispose      = gupnp_context_manager_dispose;
 
-        g_type_class_add_private (klass, sizeof (GUPnPContextManagerPrivate));
-
-        /**
-         * GSSDPClient:main-context:
-         *
-         * The #GMainContext to pass to created #GUPnPContext objects. Set to
-         * NULL to use the default.
-         *
-         * Deprecated: 0.17.2: Use g_main_context_push_thread_default()
-         *             instead.
-         **/
-        g_object_class_install_property
-                (object_class,
-                 PROP_MAIN_CONTEXT,
-                 g_param_spec_pointer
-                         ("main-context",
-                          "Main context",
-                          "GMainContext to pass to created GUPnPContext"
-                          " objects",
-                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
-                          G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |
-                          G_PARAM_STATIC_BLURB));
-
         /**
          * GUPnPContextManager:port:
          *
@@ -446,6 +426,26 @@ gupnp_context_manager_class_init (GUPnPContextManagerClass *klass)
                                     G_PARAM_STATIC_NAME |
                                     G_PARAM_STATIC_NICK |
                                     G_PARAM_STATIC_BLURB));
+        /**
+         * GUPnPContextManager:family:
+         *
+         * The socket family to create contexts for. Use %G_SOCKET_FAMILY_INVALID
+         * for any or %G_SOCKET_FAMILY_IPV4 for IPv4 contexts or
+         * %G_SOCKET_FAMILY_IPV6 for IPv6 contexts
+         *
+         * Since: 1.1.0
+         **/
+        g_object_class_install_property
+                (object_class,
+                 PROP_SOCKET_FAMILY,
+                 g_param_spec_enum ("family",
+                                    "Address family",
+                                    "Address family to create contexts for",
+                                    G_TYPE_SOCKET_FAMILY,
+                                    G_SOCKET_FAMILY_INVALID,
+                                    G_PARAM_READWRITE |
+                                    G_PARAM_CONSTRUCT_ONLY |
+                                    G_PARAM_STATIC_STRINGS));
 
          /**
          * GUPnPContextManager:white-list:
@@ -477,8 +477,7 @@ gupnp_context_manager_class_init (GUPnPContextManagerClass *klass)
                                             GUPNP_TYPE_CONTEXT_MANAGER,
                                             G_SIGNAL_RUN_FIRST,
                                             G_CALLBACK (on_context_available),
-                                            NULL, NULL,
-                                            g_cclosure_marshal_VOID__OBJECT,
+                                            NULL, NULL, NULL,
                                             G_TYPE_NONE,
                                             1,
                                             GUPNP_TYPE_CONTEXT);
@@ -497,38 +496,12 @@ gupnp_context_manager_class_init (GUPnPContextManagerClass *klass)
                                          GUPNP_TYPE_CONTEXT_MANAGER,
                                          G_SIGNAL_RUN_FIRST,
                                          G_CALLBACK (on_context_unavailable),
-                                         NULL, NULL,
-                                         g_cclosure_marshal_VOID__OBJECT,
+                                         NULL, NULL, NULL,
                                          G_TYPE_NONE,
                                          1,
                                          GUPNP_TYPE_CONTEXT);
 }
 
-/**
- * gupnp_context_manager_new:
- * @main_context: (allow-none): Deprecated: 0.17.2: %NULL. If you want to use
- *                a different main context use
- *                g_main_context_push_thread_default() instead.
- * @port: Port to create contexts for, or 0 if you don't care what port is used.
- *
- * Same as gupnp_context_manager_create().
- *
- * Returns: (transfer full): A new #GUPnPContextManager object.
- * Since: 0.13.0
- * Deprecated: 0.17.2: Use gupnp_context_manager_create().
- **/
-GUPnPContextManager *
-gupnp_context_manager_new (GMainContext *main_context,
-                           guint         port)
-{
-    if (main_context)
-            g_warning ("gupnp_context_manager_new::main_context is"
-                       " deprecated. Use "
-                       " g_main_context_push_thread_default() instead");
-
-    return gupnp_context_manager_create (port);
-}
-
 #ifdef HAVE_LINUX_RTNETLINK_H
 #include "gupnp-linux-context-manager.h"
 #endif
@@ -542,12 +515,34 @@ gupnp_context_manager_new (GMainContext *main_context,
  * NetworkManager - on its availability during runtime. If it is not available,
  * the implementation falls back to the basic Unix context manager instead.
  *
+ * Equivalent to calling #gupnp_context_manager_create_full (%G_SOCKET_FAMILY_IPV4, port);
+ *
  * Returns: (transfer full): A new #GUPnPContextManager object.
  *
  * Since: 0.17.2
  **/
 GUPnPContextManager *
 gupnp_context_manager_create (guint port)
+{
+        return gupnp_context_manager_create_full (G_SOCKET_FAMILY_IPV4, port);
+}
+
+/**
+ * gupnp_context_manager_create_full:
+ * @family: GSocketFamily to create the context for
+ * @port: Port to create contexts for, or 0 if you don't care what port is used.
+ *
+ * Factory-method to create a new #GUPnPContextManager. The final type of the
+ * #GUPnPContextManager depends on the compile-time selection or - in case of
+ * NetworkManager - on its availability during runtime. If it is not available,
+ * the implementation falls back to the basic Unix context manager instead.
+ *
+ * Returns: (transfer full): A new #GUPnPContextManager object.
+ *
+ * Since: 1.1.0
+ **/
+GUPnPContextManager *
+gupnp_context_manager_create_full (GSocketFamily family, guint port)
 {
 #if defined(USE_NETWORK_MANAGER) || defined (USE_CONNMAN)
         GDBusConnection *system_bus;
@@ -581,7 +576,7 @@ gupnp_context_manager_create (guint port)
                 else
                     impl_type = GUPNP_TYPE_UNIX_CONTEXT_MANAGER;
 #else
-		impl_type = GUPNP_TYPE_LINUX_CONTEXT_MANAGER;
+                impl_type = GUPNP_TYPE_LINUX_CONTEXT_MANAGER;
 
 #endif
 #elif defined (HAVE_IFADDRS_H)
@@ -591,6 +586,9 @@ gupnp_context_manager_create (guint port)
 #endif
         }
 #endif /* G_OS_WIN32 */
+
+        g_debug ("Using context manager implementation %s",
+                 g_type_name (impl_type));
         impl = GUPNP_CONTEXT_MANAGER (g_object_new (impl_type,
                                                     "port", port,
                                                     NULL));
@@ -616,10 +614,12 @@ void
 gupnp_context_manager_rescan_control_points (GUPnPContextManager *manager)
 {
         GList *l;
+        GUPnPContextManagerPrivate *priv;
 
         g_return_if_fail (GUPNP_IS_CONTEXT_MANAGER (manager));
 
-        l = manager->priv->objects;
+        priv = gupnp_context_manager_get_instance_private (manager);
+        l = priv->objects;
 
         while (l) {
                 if (GUPNP_IS_CONTROL_POINT (l->data)) {
@@ -649,10 +649,13 @@ void
 gupnp_context_manager_manage_control_point (GUPnPContextManager *manager,
                                             GUPnPControlPoint   *control_point)
 {
+        GUPnPContextManagerPrivate *priv;
+
         g_return_if_fail (GUPNP_IS_CONTEXT_MANAGER (manager));
         g_return_if_fail (GUPNP_IS_CONTROL_POINT (control_point));
 
-        manager->priv->objects = g_list_append (manager->priv->objects,
+        priv = gupnp_context_manager_get_instance_private (manager);
+        priv->objects = g_list_append (priv->objects,
                                                 g_object_ref (control_point));
 }
 
@@ -673,11 +676,14 @@ void
 gupnp_context_manager_manage_root_device (GUPnPContextManager *manager,
                                           GUPnPRootDevice     *root_device)
 {
+        GUPnPContextManagerPrivate *priv;
+
         g_return_if_fail (GUPNP_IS_CONTEXT_MANAGER (manager));
         g_return_if_fail (GUPNP_IS_ROOT_DEVICE (root_device));
 
-        manager->priv->objects = g_list_append (manager->priv->objects,
-                                                g_object_ref (root_device));
+        priv = gupnp_context_manager_get_instance_private (manager);
+        priv->objects = g_list_append (priv->objects,
+                                       g_object_ref (root_device));
 }
 
 /**
@@ -692,9 +698,13 @@ gupnp_context_manager_manage_root_device (GUPnPContextManager *manager,
 guint
 gupnp_context_manager_get_port (GUPnPContextManager *manager)
 {
+        GUPnPContextManagerPrivate *priv;
+
         g_return_val_if_fail (GUPNP_IS_CONTEXT_MANAGER (manager), 0);
 
-        return manager->priv->port;
+        priv = gupnp_context_manager_get_instance_private (manager);
+
+        return priv->port;
 }
 
 /**
@@ -709,7 +719,24 @@ gupnp_context_manager_get_port (GUPnPContextManager *manager)
 GUPnPWhiteList *
 gupnp_context_manager_get_white_list (GUPnPContextManager *manager)
 {
+        GUPnPContextManagerPrivate *priv;
+
         g_return_val_if_fail (GUPNP_IS_CONTEXT_MANAGER (manager), NULL);
 
-        return manager->priv->white_list;
+        priv = gupnp_context_manager_get_instance_private (manager);
+
+        return priv->white_list;
+}
+
+GSocketFamily
+gupnp_context_manager_get_socket_family (GUPnPContextManager *manager)
+{
+        GUPnPContextManagerPrivate *priv;
+
+        g_return_val_if_fail (GUPNP_IS_CONTEXT_MANAGER (manager),
+                              G_SOCKET_FAMILY_INVALID);
+
+        priv = gupnp_context_manager_get_instance_private (manager);
+
+        return priv->family;
 }
