@@ -47,6 +47,7 @@
 #include <sys/socket.h>
 #include <linux/netlink.h>
 #include <linux/rtnetlink.h>
+#include <net/if.h>
 #ifdef HAVE_LINUX_WIRELESS_H
 #include <linux/wireless.h>
 #endif
@@ -259,6 +260,8 @@ network_device_update_essid (NetworkInterface *device)
                 device->essid = g_strdup (essid);
         else
                 old_essid = NULL;
+#else
+        old_essid = NULL;
 #endif
         g_free (old_essid);
 }
@@ -267,7 +270,8 @@ static void
 network_device_create_context (NetworkInterface *device,
                                const char       *address,
                                const char       *label,
-                               const char       *mask)
+                               const char       *mask,
+                               GInetAddressMask *host_mask)
 {
         guint port;
         GError *error = NULL;
@@ -292,6 +296,7 @@ network_device_create_context (NetworkInterface *device,
                                   "interface", label,
                                   "network", device->essid ? device->essid
                                                            : mask,
+                                  "host-mask", host_mask,
                                   "port", port,
                                   NULL);
 
@@ -392,7 +397,8 @@ static void create_context (GUPnPLinuxContextManager *self,
                             const char               *label,
                             const char               *address,
                             const char               *mask,
-                            struct ifaddrmsg         *ifa);
+                            struct ifaddrmsg         *ifa,
+                            GInetAddressMask         *host_mask);
 static void remove_context (GUPnPLinuxContextManager *self,
                             const char               *address,
                             const char               *label,
@@ -439,16 +445,20 @@ extract_info (struct nlmsghdr *header,
                         *label = g_strdup ((char *) RTA_DATA (rt_attr));
                 } else if (rt_attr->rta_type == IFA_ADDRESS) {
                         if (address != NULL) {
-                                const char *result = NULL;
-                                result = inet_ntop (family,
-                                           RTA_DATA (rt_attr),
-                                           buf,
-                                           sizeof (buf));
-                                if (result == NULL) {
-                                    g_critical ("Failed ntop: %m");
+                                GInetAddress *addr;
+
+                                *address = NULL;
+                                addr = g_inet_address_new_from_bytes (RTA_DATA (rt_attr), family);
+                                if (family == AF_INET) {
+                                        *address = g_inet_address_to_string (addr);
                                 } else {
-                                    *address = g_strdup (result);
+                                        if (g_inet_address_get_is_link_local (addr) ||
+                                            g_inet_address_get_is_loopback (addr) ||
+                                            g_inet_address_get_is_site_local (addr)) {
+                                                *address = g_inet_address_to_string (addr);
+                                        }
                                 }
+                                g_object_unref (addr);
                         }
 
                         if (mask != NULL) {
@@ -524,7 +534,8 @@ create_context (GUPnPLinuxContextManager *self,
                 const char               *address,
                 const char               *label,
                 const char               *mask,
-                struct ifaddrmsg         *ifa)
+                struct ifaddrmsg         *ifa,
+                GInetAddressMask         *host_mask)
 {
         NetworkInterface *device;
         GUPnPLinuxContextManagerPrivate *priv;
@@ -548,7 +559,8 @@ create_context (GUPnPLinuxContextManager *self,
         network_device_create_context (device,
                                        address,
                                        label,
-                                       mask);
+                                       mask,
+                                       host_mask);
 }
 
 static void
@@ -819,11 +831,6 @@ receive_netlink_message (GUPnPLinuxContextManager *self, GError **error)
 
                                 g_debug ("Received RTM_NEWADDR");
                                 ifa = NLMSG_DATA (header);
-                                if (ifa->ifa_family == AF_INET6 &&
-                                        (ifa->ifa_scope != RT_SCOPE_SITE &&
-                                         ifa->ifa_scope != RT_SCOPE_LINK &&
-                                         ifa->ifa_scope != RT_SCOPE_HOST))
-                                        break;
 
                                 extract_info (header,
                                               priv->dump_netlink_packets,
@@ -833,8 +840,19 @@ receive_netlink_message (GUPnPLinuxContextManager *self, GError **error)
                                               &label,
                                               &mask);
 
-                                if (address != NULL)
-                                        create_context (self, address, label, mask, ifa);
+                                if (address != NULL) {
+                                        GInetAddress *mask_addr;
+                                        GInetAddressMask *host_mask;
+
+                                        mask_addr = g_inet_address_new_from_string (mask);
+                                        host_mask = g_inet_address_mask_new (mask_addr,
+                                                                             ifa->ifa_prefixlen,
+                                                                             NULL);
+                                        create_context (self, address, label, mask, ifa, host_mask);
+
+                                        g_object_unref (host_mask);
+                                        g_object_unref (mask_addr);
+                                }
                                 g_free (label);
                                 g_free (address);
                                 g_free (mask);
